#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <libgnome/libgnome.h>

#include "gm-world.h"
#include "gm-support.h"
#include "gm-pixbuf.h"

#define G_WORLD_INFO_XML PACKAGE_DATA_DIR "/" PACKAGE "/ui/gm-world-info.glade"

gboolean on_gm_world_info_dialog_url_button_release(GtkWidget *button, 
		GdkEventButton *event, gchar *link);

/* Private */
void
gm_world_info_dialog_set_label(GtkLabel *label, gchar *text) {
  if (text) {
    gtk_label_set_markup(label, text);
  } else {
    gtk_label_set_markup(label, _("<i>unspecified</i>"));
  }
}

/* Public */

GtkDialog *
gm_world_info_dialog_new(GmWorldInfo world_info) {
  GladeXML *xml;
  GtkWidget *dlg, *widget;
  gchar *tmp;
  
  xml = glade_xml_new(G_WORLD_INFO_XML, "dlgWorldInfo", NULL);
  dlg = glade_xml_get_widget(xml, "dlgWorldInfo");
  
  gm_world_info_dialog_set_label(GTK_LABEL(glade_xml_get_widget(xml, 
  		"lblName")), world_info.admin);
  gm_world_info_dialog_set_label(GTK_LABEL(glade_xml_get_widget(xml, 
  		"lblLocation")), world_info.location);
  gm_world_info_dialog_set_label(GTK_LABEL(glade_xml_get_widget(xml, 
  		"lblSystem")), world_info.system);
  gm_world_info_dialog_set_label(GTK_LABEL(glade_xml_get_widget(xml, 
  		"lblCharset")), world_info.charset);
  gm_world_info_dialog_set_label(GTK_LABEL(glade_xml_get_widget(xml, 
  		"lblLanguage")), world_info.language);
  
  if (world_info.homepage) {
    tmp = g_strconcat("<u><span color=\"#0000ff\">", world_info.homepage, 
    		"</span></u>", NULL);
    glade_xml_signal_connect_data(xml, 
    		"on_gm_world_info_dialog_url_button_release",
    		G_CALLBACK(on_gm_world_info_dialog_url_button_release), NULL);
    widget = glade_xml_get_widget(xml, "lblHomepage");
    gtk_label_set_markup(GTK_LABEL(widget), tmp);
    g_free(tmp);
  }
  
  if (world_info.contact) {
    tmp = g_strconcat("<u><span color=\"#0000ff\">", world_info.contact, 
    		"</span></u>", NULL);
    glade_xml_signal_connect_data(xml, 
    		"on_gm_world_info_dialog_url_button_release", 
    		G_CALLBACK(on_gm_world_info_dialog_url_button_release), "mailto:");
    widget = glade_xml_get_widget(xml, "lblEmail");
    gtk_label_set_markup(GTK_LABEL(widget), tmp);
    g_free(tmp);
  }
  
  if (world_info.logo) {
    gtk_image_set_from_pixbuf(GTK_IMAGE(glade_xml_get_widget(xml, 
    		"imageLogo")), gm_pixbuf_get(world_info.logo));
  }
  
  glade_xml_signal_connect(xml, "on_dlgWorldInfo_delete", 
  		G_CALLBACK(gtk_widget_destroy));
  glade_xml_signal_connect_data(xml, "on_buttonClose_clicked", 
  		G_CALLBACK(gm_widget_destroy_data), dlg);
  
  gtk_widget_show_all(dlg);
  
  g_object_unref(xml);
  return GTK_DIALOG(dlg);
}

/* Callbacks */

gboolean
on_gm_world_info_dialog_url_button_release(GtkWidget *label, 
		GdkEventButton *event, gchar *prefix) {
  GError *err = NULL;
  gchar *tmp, *link;
  
  if (prefix != NULL) {
  	link = g_strconcat(prefix, gtk_label_get_text(GTK_LABEL(label)), NULL);
  } else {
  	link = g_strdup(gtk_label_get_text(GTK_LABEL(label)));
  }
  
  if (!gnome_url_show(link, &err)) {
    tmp = g_strdup_printf("Could not open link: %s", err->message);
    gm_error_dialog(tmp, NULL);
    g_free(tmp);
    g_error_free(err);
  }
  
	g_free(link);  
  return FALSE;
}
